package com.landray.kmss.sys.attachment.util;

import com.landray.kmss.sys.restservice.server.model.SysRestserviceServerMain;
import com.landray.kmss.sys.mobile.util.MobileUtil;
import com.landray.kmss.util.SignUtil;
import com.landray.kmss.sys.restservice.server.model.SysRestserviceServerPolicy;
import com.landray.kmss.util.ArrayUtil;
import com.landray.kmss.sys.restservice.server.service.ISysRestserviceServerMainService;
import com.landray.kmss.sys.config.util.LicenseUtil;
import java.io.OutputStream;
import org.apache.commons.io.IOUtils;
import com.landray.kmss.sys.attachment.io.IOUtil;
import com.landray.kmss.sys.filestore.service.ISysAttUploadService;
import javax.servlet.http.HttpServletResponse;
import com.landray.kmss.sys.filestore.location.interfaces.ISysFileLocationDirectService;
import com.landray.kmss.sys.filestore.model.SysAttFile;
import com.landray.kmss.sys.attachment.service.ISysAttThirdService;
import com.landray.kmss.sys.filestore.location.util.SysFileLocationUtil;
import java.util.Map;
import com.landray.kmss.sys.attachment.model.SysAttMain;
import com.landray.kmss.util.SpringBeanUtil;
import javax.servlet.http.HttpServletRequest;
import sun.misc.BASE64Encoder;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.BufferedInputStream;
import java.io.InputStream;
import org.apache.commons.beanutils.PropertyUtils;
import com.landray.kmss.sys.config.dict.SysDataDict;
import com.landray.kmss.util.StringUtil;
import com.landray.kmss.util.ModelUtil;
import com.landray.kmss.util.ResourceUtil;
import java.util.Iterator;
import com.landray.kmss.framework.service.plugin.Plugin;
import java.util.ArrayList;
import org.slf4j.LoggerFactory;
import com.landray.kmss.sys.attachment.service.ISysAttMainCoreInnerService;
import com.landray.kmss.framework.plugin.core.config.IExtension;
import com.landray.kmss.sys.cache.KmssCache;
import com.landray.kmss.sys.cache.CacheConfig;
import java.util.List;
import org.slf4j.Logger;

public class SysAttUtil
{
    private static final Logger logger;
    private static final List<String> OFFICE_TYPES;
    private static final List<String> WPSOFFICE_TYPES;
    private static final List<String> CAD_TYPES;
    private static CacheConfig config;
    public static KmssCache tokenCache;
    private static IExtension[] extensions;
    private static ISysAttMainCoreInnerService sysAttMainService;
    
    static {
        logger = LoggerFactory.getLogger(SysAttUtil.class);
        OFFICE_TYPES = new ArrayList<String>();
        WPSOFFICE_TYPES = new ArrayList<String>();
        CAD_TYPES = new ArrayList<String>();
        SysAttUtil.config = CacheConfig.get((Class)SysAttUtil.class);
        SysAttUtil.tokenCache = new KmssCache((Class)SysAttUtil.class, SysAttUtil.config.setCacheType(3));
        SysAttUtil.OFFICE_TYPES.add("doc");
        SysAttUtil.OFFICE_TYPES.add("docx");
        SysAttUtil.OFFICE_TYPES.add("xls");
        SysAttUtil.OFFICE_TYPES.add("xlsx");
        SysAttUtil.OFFICE_TYPES.add("ppt");
        SysAttUtil.OFFICE_TYPES.add("pptx");
        SysAttUtil.OFFICE_TYPES.add("pdf");
        SysAttUtil.WPSOFFICE_TYPES.add("doc");
        SysAttUtil.WPSOFFICE_TYPES.add("docx");
        SysAttUtil.WPSOFFICE_TYPES.add("wps");
        SysAttUtil.WPSOFFICE_TYPES.add("xls");
        SysAttUtil.WPSOFFICE_TYPES.add("xlsx");
        SysAttUtil.WPSOFFICE_TYPES.add("et");
        SysAttUtil.WPSOFFICE_TYPES.add("ppt");
        SysAttUtil.WPSOFFICE_TYPES.add("pptx");
        SysAttUtil.WPSOFFICE_TYPES.add("dps");
        SysAttUtil.WPSOFFICE_TYPES.add("pdf");
        SysAttUtil.CAD_TYPES.add("dwg");
        SysAttUtil.CAD_TYPES.add("dwf");
        SysAttUtil.CAD_TYPES.add("dxf");
        SysAttUtil.extensions = Plugin.getExtensions("com.landray.kmss.sys.attachment.third.service", "*", "service");
    }
    
    private static IExtension getExtensionByConvert(final String extensionConvert) {
        if (SysAttUtil.extensions != null) {
            IExtension[] extensions;
            for (int length = (extensions = SysAttUtil.extensions).length, i = 0; i < length; ++i) {
                final IExtension extension = extensions[i];
                final String convert = Plugin.getParamValueString(extension, "convert");
                if (extensionConvert.equals(convert)) {
                    return extension;
                }
            }
        }
        SysAttUtil.logger.warn("没有获取到【point_id={},convert={}】的扩展点", "com.landray.kmss.sys.attachment.third.service", extensionConvert);
        return null;
    }
    
    public static final boolean isOfficeType(String type) {
        type = type.toLowerCase();
        for (final String imgType : SysAttUtil.OFFICE_TYPES) {
            if (imgType.equals(type)) {
                return true;
            }
        }
        return false;
    }
    
    public static final boolean isOfficeTypeByWps(String type) {
        type = type.toLowerCase();
        for (final String imgType : SysAttUtil.WPSOFFICE_TYPES) {
            if (imgType.equals(type)) {
                return true;
            }
        }
        return false;
    }
    
    public static final boolean isCADType(String type) {
        type = type.toLowerCase();
        for (final String imgType : SysAttUtil.CAD_TYPES) {
            if (imgType.equals(type)) {
                return true;
            }
        }
        return false;
    }
    
    public static boolean isSupportAttLarge() {
        final String isSupportAttLarge = ResourceUtil.getKmssConfigString("sys.att.useBigAtt");
        return isSupportAttLarge != null && "true".equals(isSupportAttLarge.trim()) && false;
    }
    
    public static String getSubjectByModel(final Object mainModel, final String defaultName) {
        String filename = defaultName;
        if (mainModel != null) {
            String modelInfo = ModelUtil.getModelClassName(mainModel);
            if (StringUtil.isNotNull(modelInfo)) {
                modelInfo = SysDataDict.getInstance().getModel(modelInfo).getDisplayProperty();
                if (StringUtil.isNotNull(modelInfo)) {
                    if ("fdId".equals(modelInfo)) {
                        return "";
                    }
                    try {
                        filename = String.valueOf(PropertyUtils.getProperty(mainModel, modelInfo));
                    }
                    catch (final Exception e) {
                        try {
                            SysAttUtil.logger.warn("获取主文档标题出错,后续尝试默认属性docSubject:", e);
                            filename = String.valueOf(PropertyUtils.getProperty(mainModel, "docSubject"));
                        }
                        catch (final Exception ex) {
                            try {
                                SysAttUtil.logger.warn("获取主文档标题出错,后续尝试属性fdName:", ex);
                                filename = String.valueOf(PropertyUtils.getProperty(mainModel, "fdName"));
                            }
                            catch (final Exception ex2) {}
                        }
                    }
                }
            }
        }
        return filename;
    }
    
    public static String getCharset(final InputStream in) throws IOException {
        String charset = "GBK";
        final byte[] first3Bytes = new byte[3];
        try {
            boolean checked = false;
            final BufferedInputStream bis = new BufferedInputStream(in);
            bis.mark(0);
            int read = bis.read(first3Bytes, 0, 3);
            if (read == -1) {
                return charset;
            }
            if (first3Bytes[0] == -1 && first3Bytes[1] == -2) {
                charset = "UTF-16LE";
                checked = true;
            }
            else if (first3Bytes[0] == -2 && first3Bytes[1] == -1) {
                charset = "UTF-16BE";
                checked = true;
            }
            else if (first3Bytes[0] == -17 && first3Bytes[1] == -69 && first3Bytes[2] == -65) {
                charset = "UTF-8";
                checked = true;
            }
            bis.reset();
            if (!checked) {
                while ((read = bis.read()) != -1) {
                    if (read >= 240) {
                        break;
                    }
                    if (128 <= read && read <= 191) {
                        break;
                    }
                    if (192 <= read && read <= 223) {
                        read = bis.read();
                        if (128 <= read && read <= 191) {
                            continue;
                        }
                        break;
                    }
                    else {
                        if (224 > read || read > 239) {
                            continue;
                        }
                        read = bis.read();
                        if (128 > read || read > 191) {
                            break;
                        }
                        read = bis.read();
                        if (128 <= read && read <= 191) {
                            charset = "UTF-8";
                            break;
                        }
                        break;
                    }
                }
            }
            bis.close();
        }
        catch (final Exception e) {
            e.printStackTrace();
        }
        return charset;
    }
    
    public static String imageToBase64Str(final String imgFile) {
        InputStream inputStream = null;
        byte[] data = null;
        Label_0122: {
            try {
                inputStream = new FileInputStream(imgFile);
                data = new byte[inputStream.available()];
                while (inputStream.read(data) > 0) {}
            }
            catch (final IOException e) {
                e.printStackTrace();
                if (inputStream != null) {
                    try {
                        inputStream.close();
                    }
                    catch (final Exception e2) {
                        SysAttUtil.logger.error("error:", e2);
                    }
                }
                break Label_0122;
            }
            finally {
                if (inputStream != null) {
                    try {
                        inputStream.close();
                    }
                    catch (final Exception e2) {
                        SysAttUtil.logger.error("error:", e2);
                    }
                }
            }
            if (inputStream != null) {
                try {
                    inputStream.close();
                }
                catch (final Exception e2) {
                    SysAttUtil.logger.error("error:", e2);
                }
            }
        }
        final BASE64Encoder encoder = new BASE64Encoder();
        return encoder.encode(data);
    }
    
    public static String imageToBase64(final HttpServletRequest request) {
        InputStream inputStream = null;
        byte[] data = null;
        Label_0132: {
            try {
                inputStream = SysAttViewerUtil.getWaterMarkPNG(SysAttViewerUtil.getWaterMarkConfigInRequest(request, "get"));
                data = new byte[inputStream.available()];
                while (inputStream.read(data) > 0) {}
            }
            catch (final Exception e) {
                SysAttUtil.logger.error("error:", e);
                if (inputStream != null) {
                    try {
                        inputStream.close();
                    }
                    catch (final Exception e2) {
                        SysAttUtil.logger.error("error:", e2);
                    }
                }
                break Label_0132;
            }
            finally {
                if (inputStream != null) {
                    try {
                        inputStream.close();
                    }
                    catch (final Exception e2) {
                        SysAttUtil.logger.error("error:", e2);
                    }
                }
            }
            if (inputStream != null) {
                try {
                    inputStream.close();
                }
                catch (final Exception e2) {
                    SysAttUtil.logger.error("error:", e2);
                }
            }
        }
        final BASE64Encoder encoder = new BASE64Encoder();
        return encoder.encode(data);
    }
    
    private static ISysAttMainCoreInnerService getSysAttMainService() {
        if (SysAttUtil.sysAttMainService == null) {
            SysAttUtil.sysAttMainService = (ISysAttMainCoreInnerService)SpringBeanUtil.getBean("sysAttMainService");
        }
        return SysAttUtil.sysAttMainService;
    }
    
    public static String generateDownloadUrl(final SysAttMain sysAttMain, final String convert, final Map<String, String> additionInfo) {
        String downloadUrl = "";
        if (sysAttMain == null) {
            SysAttUtil.logger.info("附件信息为空无法生成链接");
            return "";
        }
        SysAttFile file;
        try {
            file = getSysAttMainService().getFile(sysAttMain.getFdId());
        }
        catch (final Exception e) {
            SysAttUtil.logger.error("获取文件信息SysAttFile失败[sysAttMainId={}]", sysAttMain.getFdId(), e);
            return downloadUrl;
        }
        final ISysFileLocationDirectService directService = SysFileLocationUtil.getDirectService(file.getFdAttLocation());
        final IExtension extension = getExtensionByConvert(convert);
        boolean isSupportServiceDirect = directService.isSupportServiceDirect();
        if (extension != null) {
            final Boolean isSupportDirect = (Boolean)Plugin.getParamValue(extension, "isSupportDirect");
            isSupportServiceDirect = (isSupportServiceDirect && isSupportDirect && directService.isSupportDirect(""));
        }
        if (isSupportServiceDirect) {
            try {
                downloadUrl = directService.getDownloadUrl(file.getFdFilePath(), sysAttMain.getFdFileName());
            }
            catch (final Exception e2) {
                SysAttUtil.logger.error("获取直连链接失败[sysAttMainId={}]", sysAttMain.getFdId(), e2);
            }
            if (SysAttUtil.logger.isDebugEnabled()) {
                SysAttUtil.logger.debug("获取到的直连信息[sysAttMainId = {}, convert = {}, directUrl = {}]", sysAttMain.getFdId(), convert, downloadUrl);
            }
            return downloadUrl;
        }
        if (extension == null) {
            return null;
        }
        final ISysAttThirdService thirdService = (ISysAttThirdService)Plugin.getParamValue(extension, "bean");
        downloadUrl = thirdService.generateDownloadUrl(sysAttMain, (Map)additionInfo);
        if (SysAttUtil.logger.isDebugEnabled()) {
            SysAttUtil.logger.debug("获取到的代理链接信息[sysAttMainId = {}, convert = {}, proxyUrl = {}]", sysAttMain.getFdId(), convert, downloadUrl);
        }
        return downloadUrl;
    }
    
    public static void outputFile(final HttpServletResponse response, final SysAttMain sysAttMain) throws Exception {
        final ISysAttUploadService sysAttUploadService = (ISysAttUploadService)SpringBeanUtil.getBean("sysAttUploadService");
        final SysAttFile attFile = sysAttUploadService.getFileById(sysAttMain.getFdFileId());
        InputStream in = null;
        OutputStream out = null;
        try {
            response.setContentType(sysAttMain.getFdContentType());
            response.setHeader("Content-Disposition", "inline;filename=\"" + new String(sysAttMain.getFdFileName().getBytes("utf-8"), "ISO8859-1") + "\"");
            if (SysAttUtil.logger.isDebugEnabled()) {
                SysAttUtil.logger.debug("请求下载文件，EKP文件所处位置:{}", attFile.getFdFilePath());
            }
            in = SysFileLocationUtil.getProxyService().readFile(attFile.getFdFilePath());
            if (in == null) {
                SysAttUtil.logger.error("文件流不存在，请求查检文件路径:{}", attFile.getFdFilePath());
            }
            out = (OutputStream)response.getOutputStream();
            IOUtil.write(in, out);
        }
        catch (final Exception e) {
            if (out != null) {
                IOUtils.closeQuietly(out);
            }
            if (in != null) {
                IOUtils.closeQuietly(in);
            }
            SysAttUtil.logger.error("Foxit下载EKP的文件出现异常:" + e.getStackTrace());
            return;
        }
        finally {
            if (out != null) {
                IOUtils.closeQuietly(out);
            }
            if (in != null) {
                IOUtils.closeQuietly(in);
            }
        }
        if (out != null) {
            IOUtils.closeQuietly(out);
        }
        if (in != null) {
            IOUtils.closeQuietly(in);
        }
    }
    
    public static Boolean viewByBrowser(final SysAttMain sysAttMain) throws Exception {
        if (SysAttConfigUtil.pdfViewByBrowser() && JgWebOffice.isPDF(sysAttMain.getFdFileName()) && "true".equals(LicenseUtil.get("license-eis-project"))) {
            return true;
        }
        return false;
    }
    
    public static String getRestSign(final String attMainId, final long expires) throws Exception {
        final String signStr = String.valueOf(expires) + ":" + attMainId;
        final ISysRestserviceServerMainService sysRestMainService = (ISysRestserviceServerMainService)SpringBeanUtil.getBean("sysRestserviceServerMainService");
        final SysRestserviceServerMain sysRestserviceServerMain = sysRestMainService.findByServiceBean("sysAttachmentRestService");
        final List<SysRestserviceServerPolicy> webPolicys = sysRestserviceServerMain.getFdPolicy();
        if (ArrayUtil.isEmpty((List)webPolicys)) {
            return "";
        }
        final SysRestserviceServerPolicy webPolicy = webPolicys.get(0);
        final String sign = SignUtil.getHMAC(String.valueOf(signStr) + ":" + webPolicy.getFdLoginId(), StringUtil.isNotNull(webPolicy.getFdPassword()) ? webPolicy.getFdPassword() : webPolicy.getFdId());
        final Long time = expires - System.currentTimeMillis();
        if (time > 1000L) {
            final int type = MobileUtil.getClientType4Att(Plugin.currentRequest());
            if (11 == MobileUtil.getClientType(Plugin.currentRequest()) || 23 == type || 24 == type) {
                SysAttUtil.tokenCache.put(sign, (Object)"5", Integer.valueOf(time.intValue()));
            }
            else {
                SysAttUtil.tokenCache.put(sign, (Object)"false", Integer.valueOf(time.intValue()));
            }
        }
        return sign;
    }
    
    public static String[] getNoFileKeys() {
        return new String[] { "_sp", "_sg", "_qz", "_hw", "historyVersionAttachment", "WPSCenterSelfAttachment", "YozoHistoryAttachment" };
    }
    
    public static String[] getHistoryKeys() {
        return new String[] { "historyVersionAttachment", "WPSCenterSelfAttachment", "YozoHistoryAttachment" };
    }
}
